use Mail::IMAPClient;
use Mail::Message;
use LWP::Simple qw/get/;
use XML::RSS;
use strict;

use constant HOST => '...';
use constant USERNAME => '...';
use constant PASSWORD => '...';

my $feeds = [
        {
                imapfolder => 'INBOX.heise_news',
                url => 'http://www.heise.de/newsticker/heise.rdf'
        },
        {
                imapfolder => 'INBOX.telepolis',
                url => 'http://www.telepolis.de/news.rdf'
        }
];

my $imap = Mail::IMAPClient->new(
                Server  => HOST,
                User    => USERNAME,
                Password=> PASSWORD,
)
        or die "Verbindungsaufbau mit ".HOST." nicht mglich: $@";

foreach my $feed (@$feeds) {
        process_feed( $imap, $feed );
}

sub process_feed {
        my $imap = shift;
        my $feed = shift;

        my $imapfolder = $feed->{ imapfolder };
        $imap->create($imapfolder) or die $!
                unless $imap->exists($imapfolder);
        $imap->select($imapfolder);

        my $rss_document = get($feed->{ url })
                or die 'Konnte die Datei nicht herunterladen';
        my $xmlrss = new XML::RSS;
        $xmlrss->parse($rss_document);

        foreach my $item (@{$xmlrss->{ items }}) {
                process_item($imap, $imapfolder, $item);
        }
}

sub process_item {
        my $imap = shift;
        my $imapfolder = shift;
        my $item = shift;

        my $subject = $item->{ title };

        my @ids = $imap->search(
                "SUBJECT ".$imap->Quote($subject)
        );
        return if (@ids > 0);

        my $bodytext =
                 $item->{ title }."<br>"
                .'<A HREF="'.$item->{ link }.'">'
                .$item->{ link }.'</A>';
        $bodytext .= "\n\n".$item->{ description }
                if $item->{ description };

        my $body = Mail::Message::Body->new(
                mime_type => 'text/html',
                transfer_encoding => '8bit',
                data => $bodytext
        );

        my $message = Mail::Message->buildFromBody(
                $body,
                From => '{RSS}',
                To   => 'reader@example.com',
                Subject => $subject
        );

        $imap->append($imapfolder,$message->string);
}
